from cosmos.dataservice.objectservice import ObjectService
from cosmos.rbac.object import AccessType
import settings

__author__ = 'Maruf Maniruzzaman'

import tornado.web
from tornado import gen
from cosmos.service.requesthandler import *
import json
from cosmos.service.utils import MongoObjectJSONEncoder

PASSWORD_HMAC_SIGNATURE = "hmac:"

class LogoutHandler(RequestHandler):
    @gen.coroutine
    def get(self):
        self.clear_cookie("user")
        self.redirect('/')


class LoginHandler(RequestHandler):
    @gen.coroutine
    def get(self):

        if self.current_user:
            self.redirect('/')

        self.render('login.html')

    @tornado.web.asynchronous
    def post(self):
        username = self.get_argument("username")
        password = self.get_argument("password")

        if not username or len(username)<1:
            data = json.loads(self.request.body)
            assert isinstance(data, dict)
            username = data.get("username")
            password = data.get("password")

        db = self.settings['db']
        object_name = settings.USER_OBJECT_NAME
        obj_serv = ObjectService()

        columns = ["username", "password", "roles"]
        query = {"username": username}

        obj_serv.find(settings.SYSTEM_USER, db, object_name, query, columns, self._on_load_response)

    def _on_load_response(self, result, error):
        if error:
            raise tornado.web.HTTPError(500, error)
        else:
            assert isinstance(result, list)
            if not result or result.__len__() != 1:
                raise tornado.web.HTTPError(401, "Unauthorized")

            password = self.get_argument("password")
            if not password or len(password)<1:
                data = json.loads(self.request.body)
                assert isinstance(data, dict)
                password = data.get("password")

            user = result[0]
            loaded_password_hash = user.get(settings.PASSWORD_COLUMN_NAME)
            validate_password(password, loaded_password_hash)
            del user["password"]
            self.set_secure_cookie(settings.USER_COOKIE_NAME, MongoObjectJSONEncoder().encode(user))

            self.redirect('/')

            self.content_type = 'application/json'
            data = '{"result":"OK"}'

            self.write(data)
            self.finish()

def validate_password(password, saved_password_hash):
    hmac_password = get_hmac_password(password)
    try:
        #FIXME: WARNING possible attack: use hmac.compare_digest(a, b) instead (https://docs.python.org/2/library/hmac.html)
        if not saved_password_hash == hmac_password:
            raise tornado.web.HTTPError(401, "Unauthorized")
    except:
        raise tornado.web.HTTPError(401, "Unauthorized")


def get_hmac_password(password):
    hmac_hex = hmac.new(settings.HMAC_KEY, password).hexdigest()
    hmac_password = "{0}{1}".format(PASSWORD_HMAC_SIGNATURE, hmac_hex)
    return hmac_password

def before_user_insert(object_name, data, access_type):
    assert object_name == settings.USER_OBJECT_NAME
    assert isinstance(data, dict)
    assert access_type == AccessType.INSERT or access_type == AccessType.UPDATE

    password = data.get(settings.PASSWORD_COLUMN_NAME)

    if not password:
        return

    if password.find(PASSWORD_HMAC_SIGNATURE) > 0:
        return

    hmac_password = get_hmac_password(password)

    data[settings.PASSWORD_COLUMN_NAME] = hmac_password


from cosmos.service import auth, servicehandler
import settings

__author__ = 'Maruf Maniruzzaman'

import tornado.ioloop
import tornado.web
import tornado.options
import tornado.template
import tornado.websocket
from cosmos.dataservice.objectservice import *
from cosmos.comet.monitor import *

application = None

def start_web_service(db, port, endpoints, login_url):
    global application
    application = tornado.web.Application(
        endpoints,
        db=db,
        login_url= login_url,
        cookie_secret=settings.COOKIE_SECRET,
        xheaders=True,
        template_path=settings.TEMPLATE_PATH,
        debug=settings.DEBUG
    )

    logging.info('Starting up server on port: {0}'.format(port))
    application.listen(port)



__author__ = 'Maruf Maniruzzaman'


import sys
import logging
import getpass

import system
from cosmos.dataservice.objectservice import *
import settings


class CommandHandler():
    def handle_command(self,command):
        if command == "new-admin":
            self.create_admin_user()
            return

        print "Unknown command"

    def get_input(self, prompt):
        input = None
        while not input or len(input)==0:
            input = raw_input(prompt).strip()

        return input

    def _admin_user_created(self, result, error):
        if error:
            logging.error("Admin user creation failed.")
            logging.error(error)
        else:
            logging.info("Admin user was created successfully.")

        sys.exit(0)

    def create_admin_user(self):
        username = self.get_input('Enter admin username: ')
        password = getpass.getpass('Enter admin password: ')
        password_re = getpass.getpass('Repeat admin password: ')
        if password != password_re:
            print "Password mismatch"
            sys.exit(1)

        email = self.get_input('Enter admin email: ')
        data = {"username":username, "password":password, "email":email, "roles":[ADMIN_USER_ROLE_SID]}
        object_service = ObjectService()
        object_service.save(SYSTEM_USER, system.db, settings.USER_OBJECT_NAME, data, self._admin_user_created)




__author__ = 'maruf'

import json

from bson import ObjectId

class MongoObjectJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, ObjectId):
            return str(o)
        return json.JSONEncoder.default(self, o)


from distutils.core import setup

setup(
    name='cosmos',
    version='0.1.0.0',
    packages=['cosmos', 'cosmos.admin', 'cosmos.comet', 'cosmos.dataservice', 'cosmos.rbac', 'cosmos.schema',
              'cosmos.service'],
    url='http://cosmosframework.com',
    license='MIT',
    author='Maruf Maniruzzaman',
    author_email='marufm@cosmosframework.com',
    description='Thin server application framework',

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Application Frameworks',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2.7'
    ],

    #@@TODO: convert app to use latest version of motor
    install_requires=['tornado', 'motor==0.1.2', 'mongolog']
)
